#ifdef _NES_MAPPER_CPP_

/////////////////////////////////////////////////////////////////////
// Mapper 20
void NES_mapper20::Reset()
{
  // Init ExSound
  parent_NES->apu->SelectExSound(4);

  FILE* fp = NULL;
  char fn[256];

  GetModuleFileName(NULL, fn, 256);
  int pt = strlen(fn);
  while(fn[pt] != '\\') pt--;
  fn[pt+1] = '\0';
  strcat(fn, "DISKSYS.ROM");

  if((fp = fopen(fn, "rb")) != NULL)
  {
    uint8 head1 = fgetc(fp);
    uint8 head2 = fgetc(fp);
    uint8 head3 = fgetc(fp);
    if(head1 == 'N' && head2 == 'E' && head3 == 'S')
    {
      fseek(fp, 0x6010, SEEK_SET);
    }
    else
    {
      fseek(fp, 0, SEEK_SET);
    }
    for(uint32 i = 0; i < 0x2000; i++)
    {
      bios[i] = fgetc(fp);
    }
    fclose(fp);
  }
  else
  {
    throw "DISKSYS.ROM not found.";
  }

  NES_6502::Context context;
  parent_NES->cpu->GetContext(&context);
  context.mem_page[3] = wram + 0x0000;
  context.mem_page[4] = wram + 0x2000;
  context.mem_page[5] = wram + 0x4000;
  context.mem_page[6] = wram + 0x6000;
  context.mem_page[7] = bios;
  parent_NES->cpu->SetContext(&context);

  for(uint32 i = 0; i < 0x40000; i++)
  {
    disk_changed[i] = 0;
  }

  // read FDS disk image
  if(ROM_banks[0] == 'F' && ROM_banks[1] == 'D' && ROM_banks[2] == 'S')
  {
    for(uint32 i = 0; i < ROM_banks[4]; i++)
    {
      for(uint32 j = 0; j < 65500; j++)
      {
        disk[0x10000*i+j] = ROM_banks[16+65500*i+j];
      }
    }
  }
  else
  {
    throw "Invalid DISK image.";
  }

  // ignore frame-IRQ for Reflect World
  if(disk[0x0F] == 0xC7 && disk[0x10] == 0x52 && disk[0x11] == 0x57 &&
     disk[0x12] == 0x44 && disk[0x13] == 0x20)
  {
    parent_NES->frame_irq_disenabled = 1;
  }

  irq_enabled = 0;
  irq_counter = 0;
  irq_latch = 0;

  disk_enabled = 1;
  head_position = 0;
  write_skip = 0;
  disk_status = 0;
  write_reg = 0;
  current_side = 1;
  last_side = 1;
  insert_wait = 0;
  read_status_count = 0;
}

uint8 NES_mapper20::MemoryReadLow(uint32 addr)
{
  uint8 val = 0x00;

  switch (addr)
  {
    case 0x4030:
      {
        val = disk_status;
        disk_status = 0;
      }
      break;

    case 0x4031:
      {
        if(current_side)
        {
          val = disk[(current_side-1)*0x10000+head_position];
          // code added by stun
          if(!(write_reg & 0x80) && head_position == 0) 
          {
            val = 0;
          }
          else
          {
            head_position++;
          }
        }
        else
        {
          val = 0;
        }
      }
      break;

    case 0x4032:
      {
        if(current_side == 0)
        {
          val = 0x47;
        }
        else if (last_side != current_side)
        {
          val = 0x47;
          // wait 1.5 sec for change disk
          if (insert_wait > 90)
          {
            last_side = current_side;
            insert_wait = 0;
            read_status_count = 0;
          }
        }
        else
        {
          // for Silviana, Koneko Monogatari
          val = (read_status_count == 0) ? 0x40 : 0x42;
          read_status_count = (read_status_count + 1) & 1;
        }
      }
      break;

    case 0x4033:
      {
        val = 0x80;
      }
      break;

    default:
      {
        val = parent_NES->apu->ExRead(addr);
      }
      break;
  }

  return val;
}

void NES_mapper20::MemoryWriteLow(uint32 addr, uint8 data)
{
  switch (addr)
  {
    case 0x4020:
      {
        irq_latch = (irq_latch & 0xFF00) | data;
      }
      break;

    case 0x4021:
      {
        irq_latch = (irq_latch & 0x00FF) | ((uint32)data << 8);
      }
      break;

    case 0x4022:
      {
        irq_counter = irq_latch;
        irq_enabled = data & 0x03;
      }
      break;

    case 0x4023:
      {
        disk_enabled = data & 0x01;
      }
      break;

    case 0x4024:
      {
        if(!(write_reg & 0x04) && disk_enabled)
        {
          if(write_skip)
          {
            write_skip--;
          }
          else
          {
            disk[(current_side-1)*0x10000+head_position-2] = data;
            disk_changed[(current_side-1)*0x10000+head_position-2] = 1;
          }
        }
      }
      break;

    case 0x4025:
      {
        if (data & 0x02)
        {
          head_position = 0;
        }
        if (!(data & 0x04))
        {
          write_skip = 2;
        }
        if (data & 0x08)
        {
          set_mirroring(NES_PPU::MIRROR_HORIZ);
        }
        else
        {
          set_mirroring(NES_PPU::MIRROR_VERT);
        }
        if ((!(data & 0x40)) && (write_reg & 0x40))
        {
          head_position -= 2;
        }
        write_reg = data;
      }
      break;

    default:
      {
        parent_NES->apu->ExWrite(addr, data);
      }
      break;
  }
}

void NES_mapper20::MemoryWriteSaveRAM(uint32 addr, uint8 data)
{
  wram[addr - 0x6000] = data;
}

void NES_mapper20::MemoryWrite(uint32 addr, uint8 data)
{
  if(addr < 0xE000)
  {
    wram[addr - 0x6000] = data;
  }
}

void NES_mapper20::HSync(uint32 scanline)
{
  if(irq_enabled)
  {
    if(irq_counter <= 113)
    {
      irq_enabled &= 0x01;
      irq_counter = irq_latch;
      disk_status |= 0x01;
      parent_NES->cpu->DoIRQ();
    }
    else
    {
      irq_counter -= 113;
    }
  }
  else if(write_reg & 0x80)
  {
    parent_NES->cpu->DoIRQ();
  }
  else
  {
    disk_status &= 0xFE;
  }
}

void NES_mapper20::VSync()
{
  if (last_side != current_side)
  {
    insert_wait++;
  }
  else
  {
    insert_wait = 0;
  }
}

void NES_mapper20::SNSS_fixup()
{
  NES_6502::Context context;
  parent_NES->cpu->GetContext(&context);
  context.mem_page[3] = wram + 0x0000;
  context.mem_page[4] = wram + 0x2000;
  context.mem_page[5] = wram + 0x4000;
  context.mem_page[6] = wram + 0x6000;
  context.mem_page[7] = bios;
  parent_NES->cpu->SetContext(&context);
}
/////////////////////////////////////////////////////////////////////

#endif
